//BST.c
#include "BST.h"
#include <stdlib.h>
#include <stdio.h>

// Declare static functions here, as needed:

static void BST_displayHelper(FILE* fp, const BSTNode* pNode, uint32_t level,
                              void (*displayFn)(FILE* fp, const BSTNode* const pNode));

static void writeIndentation(FILE* fp, uint32_t level);

//////////////////////////////////////////////////////////////////////////////////// creation fn
/** Create a proper, empty binary search tree object.
 * 
 *  Pre: compare is the name of a user-defined function satisfying the BST specification
 *       display is the name of a user-defined function satisfying the BST specification
 *       destroy is the name of a user-defined function satisfying the BST specification
 * 
 *  Returns: a BST object with NULL root and configured to use the two user-supplied
 *           functions for comparings and printing user-defined data objects stored
 *           in the tree
 */
BST BST_create(int32_t (*compare)(const BSTNode* const left, const BSTNode* const right),
              void     (*display)(FILE* fp, const BSTNode* const pD),
              void     (*destroy)(BSTNode* pNode)) {
	
   BST newTree;
   
   /***  Implementation here is up to you. ***/
   
	return newTree;
}

//////////////////////////////////////////////////////////////////////////////////// insertion fns
/** Inserts user data object into a BST, unless it duplicates an existing object.
 * 
 *  Pre:  pTree points to a proper BST object
 *        userNode points to a BSTNode in a client wrapper object
 * 
 *  Returns: true iff the insertion was performed; the implementation will not
 *           insert a new element that is equal to one that's already in the
 *           BST (according to the user-supplied comparison function)
 */
bool BST_insert(BST* const pTree, const BSTNode* const userNode) {
   
   /***  Implementation here is up to you. ***/
   
	return false;
}

//////////////////////////////////////////////////////////////////////////////////// search fns
/** Searches a proper BST for an occurence of a user data object that equals
 *  *pData (according to the user-supplied comparison function).
 * 
 *  Pre: pTree points to a proper BST object
 *       userNode points to a proper BSTNode inside a user's wrapper object
 * 
 *  Returns: pointer to BSTNode containing a matching user data object; 
 *           NULL if no match is found
 */
BSTNode* BST_find(const BST* const pTree, const BSTNode* const userNode) {
   
   /***  Implementation here is up to you. ***/
   
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////////// destruction fns
/** Deallocates all dynamic memory associated with a proper BST object.
 * 
 *  Pre:   *pTree is a proper BST object
 *  Post:  all the user payloads and payload wrappers associated with *pTree
 *             have been freed
 *         the BST object itself is NOT freed since it may or may not have
 *             been allocated dynamically; that's the responsibility of the caller
 *  
 *  Calls: Payload_destroy() to handle destruction of the user's data object
 */
void BST_destroy(BST* const pTree) {
   
   /***  Implementation here is up to you. ***/
   
}

//////////////////////////////////////////////////////////////////////////////////// display fns
/** Writes a formatted display of the contents of a proper BST.
 * 
 *  Pre: fp is open on an output device
 *       pTree points to a proper BST object
 */
void BST_display(FILE* fp, const BST* const pTree) {
	
	if ( pTree->root == NULL ) {
		fprintf(fp, "Tree is empty.\n");
		return;
	}
	
	BST_displayHelper(fp, pTree->root, 0, pTree->display);
}


static void BST_displayHelper(FILE* fp, const BSTNode* pNode, uint32_t level,
                              void (*display)(FILE* fp, const BSTNode* const pD)) {
	
	if ( pNode == NULL ) {
		writeIndentation(fp, level);
		fprintf(fp, "%c\n", '*');
		return;
	}
	
	BST_displayHelper(fp, pNode->rchild, level + 1, display);
	
	writeIndentation(fp, level);
	display(fp, pNode);
	
	BST_displayHelper(fp, pNode->lchild, level + 1, display);
}

static void writeIndentation(FILE* fp, uint32_t level) {
	
	uint32_t spaces = 10 * level;
	char fmt[100] = {0};   // want: "%3*<level>c"
	fmt[0] = '%';
	sprintf(fmt+1, "%"PRIu32"c", spaces);
	fprintf(fp, fmt, ' ');
}

