#include "PayloadWrapper.h"
#include <string.h>
#include <stdlib.h>

/** Creates a new PayloadWrapper object holding *pData.
 * 
 *  Pre:  *data is a proper Payload object
 * 
 *  Returns:  pointer to a dynamically-allocated proper PayloadWrapper
 *            that encapsulates pData
 */
PayloadWrapper* PayloadWrapper_create(Payload* pData) {
	
	PayloadWrapper* newWrapper = malloc(sizeof(PayloadWrapper));
   
	newWrapper->userdata = pData;
	newWrapper->node.lchild = NULL;
	newWrapper->node.rchild = NULL;
   
	return newWrapper;
}

/** Given a pointer to a BSTNode object, computes the address of the
 *  surrounding PayloadWrapper object.
 * 
 *  Pre:  pNode points to a proper BSTNode, contained in a proper PayloadWrapper
 * 
 *  Returns:  a pointer to the surrounding PayloadWrapper
 */
PayloadWrapper* PayloadWrapper_getPtr(const BSTNode* const pNode) {
  
   return (PayloadWrapper*) ((uint8_t*) pNode - offsetof(PayloadWrapper, node));

}

/** Compares two Payload objects by comparing their name fields.
 * 
 *  Pre: left and right point to proper Payload objects
 * 
 *  Returns:  < 0 if left->name <  right->name
 *              0 if left->name == right->name
 *            > 0 if left->name >  right->name
 */
int32_t PayloadWrapper_compare(const BSTNode* const leftNode, const BSTNode* const rightNode) {
   
   PayloadWrapper* leftWrapper  = NULL;
   PaylaodWrapper* rightWrapper = NULL;
   
   /***  Implementation here is up to you. ***/
   
	return Payload_compare(leftWrapper->userdata, rightWrapper->userdata);
}

/** Deallocates all memory associated with a PayloadWrapper object.
 */
void PayloadWrapper_destroy(BSTNode* pNode) {
   
   /***  Implementation here is up to you. ***/
   
}

/** Writes a formatted representation of a Payload object.
 * 
 *  Pre:  fp is open on an output device
 *        pNode points to a proper BSTNode inside a PayloadWrapper object
 */
void PayloadWrapper_display(FILE* fp, const BSTNode* const pNode) {
	
	const PayloadWrapper* wrapper  = PayloadWrapper_getPtr(pNode);
	Payload_display(fp, wrapper->userdata);
}



